@@ -213,10 +213,10 @@ class TestSprintf < Test::Unit::TestCase
     assert_match(/\A0\.3+\z/, sprintf("%.60f", 1/3r))
     assert_match(/\A1\.20+\z/, sprintf("%.60f", 1.2r))
 
-    0.upto(9) do |len|
-      -1.upto(9) do |prec|
+    ["", *"0".."9"].each do |len|
+      ["", *".0"..".9"].each do |prec|
         ['', '+', '-', ' ', '0', '+0', '-0', ' 0', '+ ', '- ', '+ 0', '- 0'].each do |flags|
-          fmt = "%#{flags}#{len > 0 ? len : ''}#{prec >= 0 ? ".#{prec}" : ''}f"
+          fmt = "%#{flags}#{len}#{prec}f"
           [0, 0.1, 0.01, 0.001, 1.001, 100.0, 100.001, 10000000000.0, 0.00000000001, 1/3r, 2/3r, 1.2r, 10r].each do |num|
             assert_equal(sprintf(fmt, num.to_f), sprintf(fmt, num.to_r), "sprintf(#{fmt.inspect}, #{num.inspect}.to_r)")
             assert_equal(sprintf(fmt, -num.to_f), sprintf(fmt, -num.to_r), "sprintf(#{fmt.inspect}, #{(-num).inspect}.to_r)") if num > 0
@@ -226,7 +226,7 @@ class TestSprintf < Test::Unit::TestCase
     end
 
     bug11766 = '[ruby-core:71806] [Bug #11766]'
-    assert_equal("x"*10+"     1.0", sprintf("x"*10+"%8.1f", 1r))
+    assert_equal("x"*10+"     1.0", sprintf("x"*10+"%8.1f", 1r), bug11766)
   end
 
   def test_rational_precision
@@ -334,6 +334,19 @@ class TestSprintf < Test::Unit::TestCase
     assert_equal("          0x1.000p+0", sprintf("%20.3a",  1), bug3979)
   end
 
+  def test_float_prec
+    assert_equal("5.00", sprintf("%.2f",5.005))
+    assert_equal("5.02", sprintf("%.2f",5.015))
+    assert_equal("5.02", sprintf("%.2f",5.025))
+    assert_equal("5.04", sprintf("%.2f",5.035))
+    bug12889 = '[ruby-core:77864] [Bug #12889]'
+    assert_equal("1234567892", sprintf("%.0f", 1234567891.99999))
+    assert_equal("1234567892", sprintf("%.0f", 1234567892.49999))
+    assert_equal("1234567892", sprintf("%.0f", 1234567892.50000))
+    assert_equal("1234567894", sprintf("%.0f", 1234567893.50000))
+    assert_equal("1234567892", sprintf("%.0f", 1234567892.00000), bug12889)
+  end
+
   BSIZ = 120
 
   def test_skip
