@@ -1,13 +1,14 @@
+# frozen_string_literal: true
 require 'rubygems/test_case'
 require 'rubygems/commands/build_command'
-require 'rubygems/format'
+require 'rubygems/package'
 
 class TestGemCommandsBuildCommand < Gem::TestCase
 
   def setup
     super
 
-    @gem = quick_spec 'some_gem' do |s|
+    @gem = util_spec 'some_gem' do |s|
       s.rubyforge_project = 'example'
     end
 
@@ -24,16 +25,6 @@ class TestGemCommandsBuildCommand < Gem::TestCase
     util_test_build_gem @gem, gemspec_file
   end
 
-  def test_execute_yaml
-    gemspec_file = File.join(@tempdir, @gem.spec_name)
-
-    File.open gemspec_file, 'w' do |gs|
-      gs.write @gem.to_yaml
-    end
-
-    util_test_build_gem @gem, gemspec_file
-  end
-
   def test_execute_bad_spec
     @gem.date = "2010-11-08"
 
@@ -72,7 +63,17 @@ class TestGemCommandsBuildCommand < Gem::TestCase
     assert_equal "ERROR:  Gemspec file not found: some_gem\n", @ui.error
   end
 
-  def util_test_build_gem(gem, gemspec_file)
+  def test_can_find_gemspecs_without_dot_gemspec
+    gemspec_file = File.join(@tempdir, @gem.spec_name)
+
+    File.open gemspec_file + ".gemspec", 'w' do |gs|
+      gs.write @gem.to_ruby
+    end
+
+    util_test_build_gem @gem, gemspec_file
+  end
+
+  def util_test_build_gem(gem, gemspec_file, check_licenses=true)
     @cmd.options[:args] = [gemspec_file]
 
     use_ui @ui do
@@ -87,32 +88,34 @@ class TestGemCommandsBuildCommand < Gem::TestCase
     assert_equal "  Version: 2", output.shift
     assert_equal "  File: some_gem-2.gem", output.shift
     assert_equal [], output
-    assert_equal '', @ui.error
+
+    if check_licenses
+      assert_match "WARNING:  licenses is empty", @ui.error
+    end
 
     gem_file = File.join @tempdir, File.basename(gem.cache_file)
     assert File.exist?(gem_file)
 
-    spec = Gem::Format.from_file_by_path(gem_file).spec
+    spec = Gem::Package.new(gem_file).spec
 
     assert_equal "some_gem", spec.name
     assert_equal "this is a summary", spec.summary
   end
 
   def test_execute_force
-    @gem.instance_variable_set :@required_rubygems_version, nil
-
     gemspec_file = File.join(@tempdir, @gem.spec_name)
 
+    @gem.send :remove_instance_variable, :@rubygems_version
+
     File.open gemspec_file, 'w' do |gs|
-      gs.write @gem.to_yaml
+      gs.write @gem.to_ruby
     end
 
     @cmd.options[:args] = [gemspec_file]
     @cmd.options[:force] = true
 
-    util_test_build_gem @gem, gemspec_file
+    util_test_build_gem @gem, gemspec_file, false
   end
 
-
 end
 
