@@ -1,4 +1,5 @@
 # coding: UTF-8
+# frozen_string_literal: true
 
 require 'rubygems/package/tar_test_case'
 require 'rubygems/simple_gem'
@@ -132,6 +133,43 @@ class TestGemPackage < Gem::Package::TarTestCase
     assert_equal %w[lib/code.rb], files
   end
 
+  def test_add_files_symlink
+    skip 'symlink not supported' if Gem.win_platform?
+
+    spec = Gem::Specification.new
+    spec.files = %w[lib/code.rb lib/code_sym.rb]
+
+    FileUtils.mkdir_p 'lib'
+    open 'lib/code.rb',  'w' do |io| io.write '# lib/code.rb'  end
+
+    # NOTE: 'code.rb' is correct, because it's relative to lib/code_sym.rb
+    File.symlink('code.rb', 'lib/code_sym.rb')
+
+    package = Gem::Package.new 'bogus.gem'
+    package.spec = spec
+
+    tar = util_tar do |tar_io|
+      package.add_files tar_io
+    end
+
+    tar.rewind
+
+    files, symlinks = [], []
+
+    Gem::Package::TarReader.new tar do |tar_io|
+      tar_io.each_entry do |entry|
+        if entry.symlink?
+          symlinks << { entry.full_name => entry.header.linkname }
+        else
+          files << entry.full_name
+        end
+      end
+    end
+
+    assert_equal %w[lib/code.rb], files
+    assert_equal [{'lib/code_sym.rb' => 'lib/code.rb'}], symlinks
+  end
+
   def test_build
     spec = Gem::Specification.new 'build', '1'
     spec.summary = 'build'
@@ -298,7 +336,7 @@ class TestGemPackage < Gem::Package::TarTestCase
     assert_equal %w[lib/code.rb], reader.contents
   end
 
-  def test_build_signed_encryped_key
+  def test_build_signed_encrypted_key
     skip 'openssl is missing' unless defined?(OpenSSL::SSL)
 
     spec = Gem::Specification.new 'build', '1'
@@ -396,6 +434,52 @@ class TestGemPackage < Gem::Package::TarTestCase
                  "#{@destination} is not allowed", e.message)
   end
 
+  def test_extract_tar_gz_symlink_relative_path
+    skip 'symlink not supported' if Gem.win_platform?
+
+    package = Gem::Package.new @gem
+
+    tgz_io = util_tar_gz do |tar|
+      tar.add_file    'relative.rb', 0644 do |io| io.write 'hi' end
+      tar.mkdir       'lib',         0755
+      tar.add_symlink 'lib/foo.rb', '../relative.rb', 0644
+    end
+
+    package.extract_tar_gz tgz_io, @destination
+
+    extracted = File.join @destination, 'lib/foo.rb'
+    assert_path_exists extracted
+    assert_equal '../relative.rb',
+                 File.readlink(extracted)
+    assert_equal 'hi',
+                 File.read(extracted)
+  end
+
+  def test_extract_symlink_parent
+   skip 'symlink not supported' if Gem.win_platform?
+
+   package = Gem::Package.new @gem
+
+   tgz_io = util_tar_gz do |tar|
+     tar.mkdir       'lib',               0755
+     tar.add_symlink 'lib/link', '../..', 0644
+     tar.add_file    'lib/link/outside.txt', 0644 do |io| io.write 'hi' end
+   end
+
+   # Extract into a subdirectory of @destination; if this test fails it writes
+   # a file outside destination_subdir, but we want the file to remain inside
+   # @destination so it will be cleaned up.
+   destination_subdir = File.join @destination, 'subdir'
+   FileUtils.mkdir_p destination_subdir
+
+   e = assert_raises Gem::Package::PathError do
+     package.extract_tar_gz tgz_io, destination_subdir
+   end
+
+   assert_equal("installing into parent path lib/link/outside.txt of " +
+                 "#{destination_subdir} is not allowed", e.message)
+  end
+
   def test_extract_tar_gz_directory
     package = Gem::Package.new @gem
 
@@ -440,10 +524,25 @@ class TestGemPackage < Gem::Package::TarTestCase
     assert_path_exists extracted
   end
 
+  if Gem.win_platform?
+    def test_extract_tar_gz_case_insensitive
+      package = Gem::Package.new @gem
+
+      tgz_io = util_tar_gz do |tar|
+        tar.add_file 'foo/file.rb', 0644 do |io| io.write 'hi' end
+      end
+
+      package.extract_tar_gz tgz_io, @destination.upcase
+
+      extracted = File.join @destination, 'foo/file.rb'
+      assert_path_exists extracted
+    end
+  end
+
   def test_install_location
     package = Gem::Package.new @gem
 
-    file = 'file.rb'
+    file = 'file.rb'.dup
     file.taint
 
     destination = package.install_location file, @destination
@@ -483,7 +582,7 @@ class TestGemPackage < Gem::Package::TarTestCase
     skip 'no File.realpath on 1.8' if RUBY_VERSION < '1.9'
     package = Gem::Package.new @gem
 
-    file = 'foo//file.rb'
+    file = 'foo//file.rb'.dup
     file.taint
 
     destination = @destination.sub '/', '//'
@@ -868,4 +967,3 @@ class TestGemPackage < Gem::Package::TarTestCase
   end
 
 end
-
