@@ -1,5 +1,8 @@
 #!./miniruby
 
+# Used by the "make install" target to install Ruby.
+# See common.mk for more details.
+
 begin
   load "./rbconfig.rb"
 rescue LoadError
@@ -19,9 +22,14 @@ require 'optparse'
 require 'optparse/shellwords'
 require 'ostruct'
 require 'rubygems'
+begin
+  require "zlib"
+rescue LoadError
+  $" << "zlib.rb"
+end
 
 STDOUT.sync = true
-File.umask(0)
+File.umask(0222)
 
 def parse_args(argv = ARGV)
   $mantype = 'doc'
@@ -39,7 +47,7 @@ def parse_args(argv = ARGV)
   $script_mode = nil
   $strip = false
   $cmdtype = (if File::ALT_SEPARATOR == '\\'
-                File.exist?("rubystub.exe") ? 'exe' : 'bat'
+                File.exist?("rubystub.exe") ? 'exe' : 'cmd'
               end)
   mflags = []
   opt = OptionParser.new
@@ -71,7 +79,7 @@ def parse_args(argv = ARGV)
   end
   opt.on('--installed-list [FILENAME]') {|name| $installed_list = name}
   opt.on('--rdoc-output [DIR]') {|dir| $rdocdir = dir}
-  opt.on('--cmd-type=TYPE', %w[bat cmd plain]) {|cmd| $cmdtype = (cmd unless cmd == 'plain')}
+  opt.on('--cmd-type=TYPE', %w[cmd plain]) {|cmd| $cmdtype = (cmd unless cmd == 'plain')}
   opt.on('--[no-]strip') {|strip| $strip = strip}
 
   opt.order!(argv) do |v|
@@ -158,11 +166,11 @@ def install(src, dest, options = {})
   super(src, d, options)
   srcs = Array(src)
   if strip
-    d = srcs.map {|src| File.join(d, File.basename(src))} if $made_dirs[dest]
+    d = srcs.map {|s| File.join(d, File.basename(s))} if $made_dirs[dest]
     strip_file(d)
   end
   if $installed_list
-    dest = srcs.map {|src| File.join(dest, File.basename(src))} if $made_dirs[dest]
+    dest = srcs.map {|s| File.join(dest, File.basename(s))} if $made_dirs[dest]
     $installed_list.puts dest
   end
 end
@@ -275,6 +283,11 @@ def with_destdir(dir)
   $destdir + dir
 end
 
+def without_destdir(dir)
+  return dir if !$destdir or $destdir.empty?
+  dir.start_with?($destdir) ? dir[$destdir.size..-1] : dir
+end
+
 def prepare(mesg, basedir, subdirs=nil)
   return unless basedir
   case
@@ -324,9 +337,8 @@ enable_shared = CONFIG["ENABLE_SHARED"] == 'yes'
 dll = CONFIG["LIBRUBY_SO", enable_shared]
 lib = CONFIG["LIBRUBY", true]
 arc = CONFIG["LIBRUBY_A", true]
-major = CONFIG["MAJOR", true]
-minor = CONFIG["MINOR", true]
-load_relative = configure_args.include?("--enable-load-relative")
+config_h = File.read(CONFIG["EXTOUT"]+"/include/"+CONFIG["arch"]+"/ruby/config.h")
+load_relative = config_h[/^\s*#\s*define\s+LOAD_RELATIVE\s+(\d+)/, 1].to_i.nonzero?
 
 install?(:local, :arch, :bin, :'bin-arch') do
   prepare "binary commands", bindir
@@ -347,7 +359,7 @@ install?(:local, :arch, :lib) do
   prepare "base libraries", libdir
 
   install lib, libdir, :mode => $prog_mode, :strip => $strip unless lib == arc
-  install arc, libdir, :mode => $data_mode
+  install arc, libdir, :mode => $data_mode unless CONFIG["INSTALL_STATIC_LIBRARY"] == "no"
   if dll == lib and dll != arc
     for link in CONFIG["LIBRUBY_ALIASES"].split
       ln_sf(dll, File.join(libdir, link))
@@ -377,6 +389,15 @@ install?(:ext, :arch, :'ext-arch') do
   install_recursive("#{$extout}/#{CONFIG['arch']}", archlibdir, :no_install => noinst, :mode => $prog_mode, :strip => $strip)
   prepare "extension objects", sitearchlibdir
   prepare "extension objects", vendorarchlibdir
+  if extso = File.read("exts.mk")[/^EXTSO[ \t]*=[ \t]*((?:.*\\\n)*.*)/, 1] and
+    !(extso = extso.gsub(/\\\n/, '').split).empty?
+    libpathenv = CONFIG["LIBPATHENV"]
+    dest = CONFIG[!libpathenv || libpathenv == "PATH" ? "bindir" : "libdir"]
+    prepare "external libraries", dest
+    for file in extso
+      install file, dest, :mode => $prog_mode
+    end
+  end
 end
 install?(:ext, :arch, :hdr, :'arch-hdr') do
   prepare "extension headers", archhdrdir
@@ -406,16 +427,16 @@ install?(:doc, :capi) do
   install_recursive "doc/capi", docdir+"/capi", :mode => $data_mode
 end
 
-if load_relative
+if load_relative or /\s/ =~ bindir
   PROLOG_SCRIPT = <<EOS
 #!/bin/sh\n# -*- ruby -*-
-bindir=`#{CONFIG["CHDIR"]} "${0%/*}" 2>/dev/null; pwd`
+bindir="#{load_relative ? '${0%/*}' : bindir.gsub(/\"/, '\\\\"')}"
 EOS
   if CONFIG["LIBRUBY_RELATIVE"] != 'yes' and libpathenv = CONFIG["LIBPATHENV"]
     pathsep = File::PATH_SEPARATOR
     PROLOG_SCRIPT << <<EOS
-prefix="${bindir%/bin}"
-export #{libpathenv}="$prefix/lib${#{libpathenv}#{pathsep}+#{pathsep}$#{libpathenv}}"
+libdir="#{load_relative ? '${bindir%/bin}/lib' : libdir.gsub(/\"/, '\\\\"')}"
+export #{libpathenv}="$libdir${#{libpathenv}:+#{pathsep}$#{libpathenv}}"
 EOS
   end
   PROLOG_SCRIPT << %Q[exec "$bindir/#{ruby_install_name}" -x "$0" "$@"\n]
@@ -457,20 +478,21 @@ install?(:local, :comm, :bin, :'bin-comm') do
   else
     trans = proc {|base| base}
   end
+  prebatch = ':""||{ ""=> %q<-*- ruby -*-'"\n"
+  postbatch = PROLOG_SCRIPT ? "};{\n#{PROLOG_SCRIPT.sub(/\A(?:#.*\n)*/, '')}" : ''
+  postbatch << ">,\n}\n"
+  postbatch.gsub!(/(?=\n)/, ' #')
   install_recursive(File.join(srcdir, "bin"), bindir, :maxdepth => 1) do |src, cmd|
     cmd = cmd.sub(/[^\/]*\z/m) {|n| RbConfig.expand(trans[n])}
 
-    shebang = ''
-    body = ''
-    open(src, "rb") do |f|
-      shebang = f.gets
-      body = f.read
+    shebang, body = open(src, "rb") do |f|
+      next f.gets, f.read
     end
     shebang or raise "empty file - #{src}"
-    if PROLOG_SCRIPT
+    if PROLOG_SCRIPT and !$cmdtype
       shebang.sub!(/\A(\#!.*?ruby\b)?/) {PROLOG_SCRIPT + ($1 || "#!ruby\n")}
     else
-      shebang.sub!(/\A\#!.*?ruby\b/) {"#!" + ruby_shebang}
+      shebang.sub!(/\A(\#!.*?ruby\b)?/) {"#!" + ruby_shebang + ($1 ? "" : "\n")}
     end
     shebang.sub!(/\r$/, '')
     body.gsub!(/\r$/, '')
@@ -480,18 +502,8 @@ install?(:local, :comm, :bin, :'bin-comm') do
       case $cmdtype
       when "exe"
         stub + shebang + body
-      when "bat"
-        [<<-"EOH".gsub(/^\s+/, ''), shebang, body, "__END__\n:endofruby\n"].join.gsub(/$/, "\r")
-          @echo off
-          @if not "%~d0" == "~d0" goto WinNT
-          #{ruby_bin} -x "#{cmd}" %1 %2 %3 %4 %5 %6 %7 %8 %9
-          @goto endofruby
-          :WinNT
-          "%~dp0#{ruby_install_name}" -x "%~f0" %*
-          @goto endofruby
-        EOH
       when "cmd"
-        <<"/EOH" << shebang << body
+        prebatch + <<"/EOH" << postbatch << shebang << body
 @"%~dp0#{ruby_install_name}" -x "%~f0" %*
 @exit /b %ERRORLEVEL%
 /EOH
@@ -523,9 +535,14 @@ install?(:local, :comm, :man) do
   mdocs = Dir["#{srcdir}/man/*.[1-9]"]
   prepare "manpages", mandir, ([] | mdocs.collect {|mdoc| mdoc[/\d+$/]}).sort.collect {|sec| "man#{sec}"}
 
+  case $mantype
+  when /\.(?:(gz)|bz2)\z/
+    compress = $1 ? "gzip" : "bzip2"
+    suffix = $&
+  end
   mandir = File.join(mandir, "man")
   has_goruby = File.exist?(goruby_install_name+exeext)
-  require File.join(srcdir, "tool/mdoc2man.rb") if $mantype != "doc"
+  require File.join(srcdir, "tool/mdoc2man.rb") if /\Adoc\b/ !~ $mantype
   mdocs.each do |mdoc|
     next unless File.file?(mdoc) and open(mdoc){|fh| fh.read(1) == '.'}
     base = File.basename(mdoc)
@@ -537,14 +554,41 @@ install?(:local, :comm, :man) do
     destname = ruby_install_name.sub(/ruby/, base.chomp(".#{section}"))
     destfile = File.join(destdir, "#{destname}.#{section}")
 
-    if $mantype == "doc"
-      install mdoc, destfile, :mode => $data_mode
+    if /\Adoc\b/ =~ $mantype
+      if compress
+        w = open(mdoc) {|f|
+          stdin = STDIN.dup
+          STDIN.reopen(f)
+          begin
+            destfile << suffix
+            IO.popen(compress) {|f| f.read}
+          ensure
+            STDIN.reopen(stdin)
+            stdin.close
+          end
+        }
+        open_for_install(destfile, $data_mode) {w}
+      else
+        install mdoc, destfile, :mode => $data_mode
+      end
     else
       class << (w = [])
         alias print push
       end
       open(mdoc) {|r| Mdoc2Man.mdoc2man(r, w)}
-      open_for_install(destfile, $data_mode) {w.join("")}
+      w = w.join("")
+      if compress
+        require 'tmpdir'
+        Dir.mktmpdir("man") {|d|
+          dest = File.join(d, File.basename(destfile))
+          File.open(dest, "wb") {|f| f.write w}
+          if system(compress, dest)
+            w = File.open(dest+suffix, "rb") {|f| f.read}
+            destfile << suffix
+          end
+        }
+      end
+      open_for_install(destfile, $data_mode) {w}
     end
   end
 end
@@ -586,7 +630,8 @@ module RbInstall
         when "ext"
           prefix = "#{$extout}/#{CONFIG['arch']}/"
           base = "#{prefix}#{relative_base}"
-          Dir.glob("#{base}{.so,/**/*.so}").collect do |built_library|
+          dlext = CONFIG['DLEXT']
+          Dir.glob("#{base}{.#{dlext},/**/*.#{dlext}}").collect do |built_library|
             remove_prefix(prefix, built_library)
           end
         when "lib"
@@ -603,59 +648,53 @@ module RbInstall
         string.sub(/\A#{Regexp.escape(prefix)}/, "")
       end
     end
+  end
 
-    class Reader < Struct.new(:src)
-      def gemspec
-        @gemspec ||= begin
-          spec = Gem::Specification.load(src) || raise("invalid spec in #{src}")
-          file_collector = FileCollector.new(File.dirname(src))
-          spec.files = file_collector.collect
-          spec
-        end
-      end
-
-      def spec_source
-        @gemspec.to_ruby
+  class UnpackedInstaller < Gem::Installer
+    module DirPackage
+      def extract_files(destination_dir, pattern = "*")
+        path = File.dirname(@gem.path)
+        return if path == destination_dir
+        File.chmod(0700, destination_dir)
+        mode = pattern == "bin/*" ? $script_mode : $data_mode
+        install_recursive(path, without_destdir(destination_dir),
+                          :glob => pattern,
+                          :no_install => "*.gemspec",
+                          :mode => mode)
+        File.chmod($dir_mode, destination_dir)
       end
     end
 
-    class Generator < Struct.new(:name, :base_dir, :src, :execs)
-      def gemspec
-        @gemspec ||= eval spec_source
-      end
+    def initialize(spec, *options)
+      super(spec.loaded_from, *options)
+      @package.extend(DirPackage).spec = spec
+    end
 
-      def spec_source
-        <<-GEMSPEC
-Gem::Specification.new do |s|
-  s.name = #{name.dump}
-  s.version = #{version.dump}
-  s.summary = "This #{name} is bundled with Ruby"
-  s.executables = #{execs.inspect}
-  s.files = #{files.inspect}
-end
-        GEMSPEC
-      end
+    def write_cache_file
+    end
 
-      private
-      def version
-        version = open(src) { |f|
-          f.find { |s| /^\s*\w*VERSION\s*=(?!=)/ =~ s }
-        } or return
-        version.split(%r"=\s*", 2)[1].strip[/\A([\'\"])(.*?)\1/, 2]
-      end
-
-      def files
-        file_collector = FileCollector.new(base_dir)
-        file_collector.collect
-      end
+    def build_extensions
     end
   end
 end
+
+class Gem::Installer
+  install = instance_method(:install)
+  define_method(:install) do
+    spec.post_install_message = nil
+    install.bind(self).call
+  end
+
+  generate_bin_script = instance_method(:generate_bin_script)
+  define_method(:generate_bin_script) do |filename, bindir|
+    generate_bin_script.bind(self).call(filename, bindir)
+    File.chmod($script_mode, File.join(bindir, formatted_program_filename(filename)))
+  end
+end
+
 # :startdoc:
 
 install?(:ext, :comm, :gem) do
-  $:.unshift(File.join(srcdir, "lib"))
-  require("rubygems.rb")
   gem_dir = Gem.default_dir
   directories = Gem.ensure_gem_subdirectories(gem_dir, :mode => $dir_mode)
   prepare "default gems", gem_dir, directories
@@ -664,70 +703,96 @@ install?(:ext, :comm, :gem) do
   default_spec_dir = "#{spec_dir}/default"
   makedirs(default_spec_dir)
 
-  gems = {}
-  File.foreach(File.join(srcdir, "defs/default_gems")) do |line|
-    line.chomp!
-    line.sub!(/\s*#.*/, '')
-    next if line.empty?
-    words = []
-    line.scan(/\G\s*([^\[\]\s]+|\[([^\[\]]*)\])/) do
-      words << ($2 ? $2.split : $1)
-    end
-    name, base_dir, src, execs = *words
-    next unless name and base_dir and src
-
-    src       = File.join(srcdir, src)
-    base_dir  = File.join(srcdir, base_dir)
-    specgen   = RbInstall::Specs::Generator.new(name, base_dir, src, execs || [])
-    gems[name] ||= specgen
-  end
-
-  Dir.glob(srcdir+"/{lib,ext}/**/*.gemspec").each do |src|
-    specgen   = RbInstall::Specs::Reader.new(src)
-    gems[specgen.gemspec.name] ||= specgen
-  end
-
-  gems.sort.each do |name, specgen|
-    gemspec   = specgen.gemspec
-    base_dir  = specgen.src.sub(/\A#{Regexp.escape(srcdir)}\//, "")
+  gems = Dir.glob(srcdir+"/{lib,ext}/**/*.gemspec").map {|src|
+    spec = Gem::Specification.load(src) || raise("invalid spec in #{src}")
+    file_collector = RbInstall::Specs::FileCollector.new(File.dirname(src))
+    files = file_collector.collect
+    next if files.empty?
+    spec.files = files
+    spec
+  }
+  gems.compact.sort_by(&:name).each do |gemspec|
     full_name = "#{gemspec.name}-#{gemspec.version}"
 
     puts "#{" "*30}#{gemspec.name} #{gemspec.version}"
     gemspec_path = File.join(default_spec_dir, "#{full_name}.gemspec")
     open_for_install(gemspec_path, $data_mode) do
-      specgen.spec_source
+      gemspec.to_ruby
     end
 
     unless gemspec.executables.empty? then
-      bin_dir = File.join(gem_dir, 'gems', full_name, 'bin')
+      bin_dir = File.join(gem_dir, 'gems', full_name, gemspec.bindir)
       makedirs(bin_dir)
 
       execs = gemspec.executables.map {|exec| File.join(srcdir, 'bin', exec)}
-      install(execs, bin_dir, :mode => $prog_mode)
+      install(execs, bin_dir, :mode => $script_mode)
     end
   end
 end
 
 install?(:ext, :comm, :gem) do
-  begin
-    require "zlib"
-  rescue LoadErroe
+  gem_dir = Gem.default_dir
+  directories = Gem.ensure_gem_subdirectories(gem_dir, :mode => $dir_mode)
+  prepare "bundle gems", gem_dir, directories
+  install_dir = with_destdir(gem_dir)
+  installed_gems = {}
+  options = {
+    :install_dir => install_dir,
+    :bin_dir => with_destdir(bindir),
+    :domain => :local,
+    :ignore_dependencies => true,
+    :dir_mode => $dir_mode,
+    :data_mode => $data_mode,
+    :prog_mode => $prog_mode,
+    :wrappers => true,
+    :format_executable => true,
+  }
+  gem_ext_dir = "#$extout/gems/#{CONFIG['arch']}"
+  extensions_dir = Gem::StubSpecification.gemspec_stub("", gem_dir, gem_dir).extensions_dir
+  Gem::Specification.each_gemspec([srcdir+'/gems/*']) do |path|
+    dir = File.dirname(path)
+    spec = Dir.chdir(dir) {
+      Gem::Specification.load(File.basename(path))
+    }
+    next unless spec.platform == Gem::Platform::RUBY
+    next unless spec.full_name == path[srcdir.size..-1][/\A\/gems\/([^\/]+)/, 1]
+    spec.extension_dir = "#{extensions_dir}/#{spec.full_name}"
+    if File.directory?(ext = "#{gem_ext_dir}/#{spec.full_name}")
+      spec.extensions[0] ||= "-"
+    end
+    ins = RbInstall::UnpackedInstaller.new(spec, options)
+    puts "#{" "*30}#{spec.name} #{spec.version}"
+    ins.install
+    File.chmod($data_mode, File.join(install_dir, "specifications", "#{spec.full_name}.gemspec"))
+    unless spec.extensions.empty?
+      install_recursive(ext, spec.extension_dir)
+    end
+    installed_gems[spec.full_name] = true
   end
+  installed_gems, gems = Dir.glob(srcdir+'/gems/*.gem').partition {|gem| installed_gems.key?(File.basename(gem, '.gem'))}
+  unless installed_gems.empty?
+    install installed_gems, gem_dir+"/cache"
+  end
+  next if gems.empty?
   if defined?(Zlib)
-    require 'pathname'
-    gem_dir = Gem.default_dir
-    directories = Gem.ensure_gem_subdirectories(gem_dir, :mode => $dir_mode)
-    prepare "bundle gems", gem_dir, directories
-    Dir.glob(srcdir+'/gems/*.gem').each do |gem|
-      Gem.install gem, Gem::Requirement.default, :install_dir => with_destdir(Gem.dir), :domain => :local, :ignore_dependencies => true
-      gemname = Pathname(gem).basename
+    Gem.instance_variable_set(:@ruby, with_destdir(File.join(bindir, ruby_install_name)))
+    silent = Gem::SilentUI.new
+    gems.each do |gem|
+      inst = Gem::Installer.new(gem, options)
+      inst.spec.extension_dir = with_destdir(inst.spec.extension_dir)
+      begin
+        Gem::DefaultUserInteraction.use_ui(silent) {inst.install}
+      rescue Gem::InstallError => e
+        next
+      end
+      gemname = File.basename(gem)
       puts "#{" "*30}#{gemname}"
     end
     # fix directory permissions
     # TODO: Gem.install should accept :dir_mode option or something
-    File.chmod($dir_mode, *Dir.glob(with_destdir(Gem.dir)+"/**/"))
+    File.chmod($dir_mode, *Dir.glob(install_dir+"/**/"))
     # fix .gemspec permissions
-    File.chmod($data_mode, *Dir.glob(with_destdir(Gem.dir)+"/specifications/*.gemspec"))
+    File.chmod($data_mode, *Dir.glob(install_dir+"/specifications/*.gemspec"))
   else
     puts "skip installing bundle gems because of lacking zlib"
   end
